/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#include "stdafx.h"
#include "..\MTuxvision.h"
#include "BouquetsReader.h"
#include "..\MT_Utils.h"
#include "..\parser.h"

#define MAX_BUFF					512

CService *MT_GetServiceByID(const char *pszID)
{
 return MT_GetApp()->m_cBouquetsReader.GetService(pszID);
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBouquetsReader::CBouquetsReader()
{
 Clear();
} 

CBouquetsReader::~CBouquetsReader()
{
 Destroy();
 Clear();
}

void CBouquetsReader::Clear()
{
 m_pFTP = NULL;
}

void CBouquetsReader::Destroy()
{
 POSITION pos;
 CService *pServ = NULL;
 CString  szKey;

 for(pos = m_aServices.GetStartPosition(); pos != NULL; )
 {
   m_aServices.GetNextAssoc( pos, szKey, (void*&)pServ);

   if(pServ)
	   delete pServ;
 }
 
 m_aServices.RemoveAll();
}

bool CBouquetsReader::Init()
{

 return true; 
}

bool CBouquetsReader::Open()
{
 char szIP[16];
 sprintf(szIP,"%i.%i.%i.%i",g_sGlobals.m_byDreamboxIP[0],g_sGlobals.m_byDreamboxIP[1],g_sGlobals.m_byDreamboxIP[2],g_sGlobals.m_byDreamboxIP[3]);
 
 try
 {
  m_pFTP = g_isDreambox.GetFtpConnection(szIP,g_sGlobals.m_szUsername,g_sGlobals.m_szPassword,g_sGlobals.m_portFtp,g_sGlobals.m_bPassiveFTP);
 }catch(CInternetException *pException)
 {
  char szError[255];

  pException->GetErrorMessage(szError,255);

  MT_PRINT(0,"CBouquetsReader::Open - Couldn't connect to dreambox FTP (%s). Cause : %s",szIP,szError);
  g_sGlobals.m_iDreamboxOnline = 0;
  return false;	
 };

 g_sGlobals.m_iDreamboxOnline = 1;

 return true;
}

void CBouquetsReader::Close()
{
 if(m_pFTP)
	 delete m_pFTP;

 m_pFTP = NULL;
}

bool CBouquetsReader::GetMasterFile(const char *pFilePath)
{
 char szLocal[512];

 if(!m_pFTP)
 {
  MT_PRINT(0,"CBouquetsReader::GetMasterFile - Loading cached %s...",pFilePath);
  FILE *p = fopen(StripFileName(pFilePath),"r");
  if(!p)
	  return false;

  fclose(p);

  return true;
 }

 // -> Copy bouquets file
 sprintf(szLocal,"%s/%s",MT_GetApp()->m_szAppPath,StripFileName(pFilePath));

 if(!m_pFTP->GetFile(pFilePath,szLocal,FALSE))
 {
  char *pMsg = MT_GetApp()->GetWin32ErrorMessage();

  MT_PRINT(0,"CBouquetsReader::GetMasterFiles - Unable to download %s",pFilePath);
  MT_PRINT(0,"\tOS Response: %s",pMsg);

  LocalFree((LPVOID)pMsg);

  return false;
 }

 return true;
}

bool CBouquetsReader::ScanTVMasterFile()
{
 char szLocal[255];

 sprintf(szLocal,"%s/%s",MT_GetApp()->m_szAppPath,MT_TV_USERBOUQUETS_FILE);

 // -> Just to be sure
 int iNumPaths = ParseUserBouquetsFile(&m_RootTVNode,szLocal);
 if(!iNumPaths)
 {
  MT_PRINT(0,"CBoquetsReader::ScanTVMasterFile - Could not parse user boquets");
  return false;
 }

 // -> Load lists.
 CBouquetNode *pNode = &m_RootTVNode;
 CBouquetNode *pChild = NULL;

 for(int k = 0; k <= pNode->m_aBouquets.GetUpperBound(); ++k)
 {
  pChild = (CBouquetNode*)pNode->m_aBouquets.GetAt(k);

 
  sprintf(szLocal,"%s/%s",g_sGlobals.m_szBouquetPath, StripFileName(pChild->m_szPath));
  GetMasterFile(pChild->m_szPath);

  sprintf(szLocal,"%s/%s",MT_GetApp()->m_szAppPath, StripFileName(pChild->m_szPath));
  ParseUserBouquetsFile(pChild,szLocal);
 }

 return true;
}

bool CBouquetsReader::ScanRadioMasterFile()
{
 char szLocal[255];

 sprintf(szLocal,"%s/%s",MT_GetApp()->m_szAppPath,MT_RADIO_USERBOUQUETS_FILE);

 // -> Just to be sure
 int iNumPaths = ParseUserBouquetsFile(&m_RootRadioNode,szLocal);
 if(!iNumPaths)
 {
  MT_PRINT(0,"CBoquetsReader::ScanRadioMasterFile - Could not parse user boquets");
  return false;
 }

 // -> Load lists.
 CBouquetNode *pNode = &m_RootRadioNode;
 CBouquetNode *pChild = NULL;

 for(int k = 0; k <= pNode->m_aBouquets.GetUpperBound(); ++k)
 {
  pChild = (CBouquetNode*)pNode->m_aBouquets.GetAt(k);

 
  sprintf(szLocal,"%s/%s",g_sGlobals.m_szBouquetPath, StripFileName(pChild->m_szPath));
  GetMasterFile(pChild->m_szPath);

  sprintf(szLocal,"%s/%s",MT_GetApp()->m_szAppPath, StripFileName(pChild->m_szPath));
  ParseUserBouquetsFile(pChild,szLocal);
 }

 return true;
}

bool CBouquetsReader::ScanBouquetsMasterFile()
{
 char szLocal[255];

 sprintf(szLocal,"%s/bouquets",MT_GetApp()->m_szAppPath);

 // -> Just to be sure
 int iNumPaths = ParseBouquetsFile(&m_BouquetNode,szLocal);
 if(!iNumPaths)
 {
  MT_PRINT(0,"CBoquetsReader::ScanBouquetsMasterFile - Could not parse boquets");
  return false;
 }

 return true;
}

int CBouquetsReader::ParseBouquetsFile(CBouquetNode *pRootNode, const char *pszFileName)
{
 FILE *p = fopen(pszFileName,"rt");
 if(!p)
	 return 0;

 char Buff[MAX_BUFF];
 int iFound					= 0;
 bool bSkipping				= true;
 bool bSkipUnknown			= true;
 CBouquetNode *pNewNode		= NULL;

 while(!feof(p))
 {
  fgets(Buff,MAX_BUFF,p);

  if(bSkipping)
  {
   if(Buff == strstr(Buff,"bouquets"))
	   bSkipping = false;

   continue;
  };

  /*
	Entries are this way:
		- a code (dont know what's for)
		- Bouquet name
		- service list for this bouquet
		- '/' char to tell pc ready for next bouquet
  */
  if(Buff == strstr(Buff,"end"))
	  break;		// -> File ended. Let's get out of this hell.

  if(Buff[0] != '/')
  {
   if(bSkipUnknown)	// Let's skip first line because we dont know what's for
   {
	bSkipUnknown = false;

	pNewNode = new CBouquetNode();
	if(!pNewNode)
	{
	 fclose(p);
	 MT_PRINT(0,"CBouquetsReader::ParseBouquetsFile -  Cannot allocate enougth memory");
	 return false;
	};
	continue;
   }

   // -> We can either find bouquet name or service string
   if(CountChrInStr(':',Buff,3) >= 3)
   {
	CBouquetNode *pChannel = new CBouquetNode();
	if(!pChannel)
	{
	 MT_PRINT(0,"CBouquetsReader::ParseBouquetsFile -  Cannot allocate enougth memory");
	 continue;
	};

	char *pServiceID = MT_GetServiceKeyByServiceString(Buff);
	char *pNewString = MT_BuildBouquetStringByServiceString(Buff);

	VERIFY(pNewString != NULL);

	CService *pFoundService = NULL;

	if(m_aServices.Lookup(pServiceID,(void*&)pFoundService))
	{
	 // -> Skip corrupted services..
	 if(!pFoundService->m_pszName)
	 {
	  MT_PRINT(0,"CBouquetsReader::ParseBouquetsFile -  There was an error while parsing %s. Corrupted service or wrong memory allocation! Check %s in your services file.",pszFileName,pNewString);

	  // -> Free memory and skip this one..
	  delete [] pServiceID;
	  delete [] pNewString;
	  delete pChannel;
	  
	  continue;
	 }

	 strcpy(pChannel->m_szName,pFoundService->m_pszName);
	 strcpy(pChannel->m_szPath,pNewString);
	 //pChannel->m_szPath[strlen(Buff) - 1] = '\0';

	 delete [] pServiceID;
	 delete [] pNewString;
	}else
	{
	 strcpy(pChannel->m_szName,Buff);
	}

	pNewNode->m_aChannels.Add(pChannel);	
   }else
   {
	strcpy(pNewNode->m_szName,Buff);
	pNewNode->m_szName[strlen(pNewNode->m_szName) - 1] = '\0';
   }
  }else
  {
   pRootNode->m_aBouquets.SetAtGrow(iFound++,pNewNode);
   bSkipUnknown = true;
  }
 }

 fclose(p);

 return iFound;
}

int CBouquetsReader::ParseUserBouquetsFile(CBouquetNode *pRootNode,const char *pszFileName)
{
 FILE *p = fopen(pszFileName,"rt");
 if(!p)
	 return 0;

 int iFound = 0;
 char Buff[MAX_BUFF];
 CBouquetNode *pNewNode		= NULL;

 while(!feof(p))
 {
  if(!fgets(Buff,MAX_BUFF,p))
	  break;	// -> So we safetly exit loop

  /*
   in tv.epl files paths are also without # so we can simply skip # lines.
   We also skip #DESCRIPTION tags..
  */
  if(Buff[0] == '#')
  {
   switch(Buff[1])
   {
   case 'N':// #NAME
	   {
		char *pName = Buff + 6;
		pName[strlen(pName) - 1] = '\0';
		strcpy(pRootNode->m_szName,pName);
	   }break;

   case 'S':// #SERVICE
	   {
		pNewNode = new CBouquetNode();
		if(!pNewNode)
			continue;

		char *pServiceName = Buff + 10;
		char *pServiceID = MT_GetServiceKeyByBouquetString(pServiceName);

		CService *pFoundService = NULL;

		if(m_aServices.Lookup(pServiceID,(void*&)pFoundService))
		{
		 strcpy(pNewNode->m_szName,pFoundService->m_pszName);
		 strcpy(pNewNode->m_szPath,pServiceName);
		 pNewNode->m_szPath[strlen(pServiceName) - 1] = '\0';

		 delete [] pServiceID;
		}else
		{
		 strcpy(pNewNode->m_szName,pServiceName);
		}

		pRootNode->m_aChannels.Add(pNewNode);
	   }break;
   }
  }else
  {
   /*
    Copy found paths in a temp array 
   */
   pNewNode = new CBouquetNode();
   if(!pNewNode)
	  continue;

   int iLastChar = strlen(Buff) - 1;
   strncpy(pNewNode->m_szPath,Buff,iLastChar);
   pNewNode->m_szPath[iLastChar] = '\0';
  
   pRootNode->m_aBouquets.SetAtGrow(iFound++,pNewNode);
  }
 }

 fclose(p);

 return iFound;
}

bool CBouquetsReader::LoadServicesFile()
{
 char szBuff[MAX_BUFF];
 unsigned int uLineNo = 0;

 sprintf(szBuff,"%s/%s",MT_GetApp()->m_szAppPath, MT_SERVICES_FILE);

 FILE *p = fopen(szBuff,"rt");
 if(!p)
	 return false;

 MT_PRINT(0,"Parsing services file...");

 bool bKeepSkipping = true;
 CService *pLastService = NULL;

 while(!feof(p))
 {
  fgets(szBuff,MAX_BUFF,p);
  ++uLineNo;

  // -> Let's skip shit untill we get to "services"
  if(bKeepSkipping)
  {
   if(szBuff == strstr(szBuff,"services"))
	   bKeepSkipping = false;

   continue;
  }

  if(szBuff == strstr(szBuff,"end"))
	  break;

  /* -> Let's do our dirty job .. 
	  We found a string with more than 4 ':' so it could be a service
	   
	  12cf:00820000:2e7c:013e:0:2862
	  
	  or some parameters. 
	  
	  c:0000a0,c:010050,c:020020,c:0300a0,p:CP

	  In the last case there should be at least a ',' (comma) so we can distinguish
  */
  if((CountChrInStr(':',szBuff,4) >= 4) && !strchr(szBuff,','))
  {
   if(pLastService && !pLastService->m_pszName)
		MT_PRINT(0,"CBouquetsReader::LoadServicesFile - Unable to find service name for %s [Near line %i]",pLastService->m_szData, uLineNo);	
	
   pLastService = CreateServiceFromString(szBuff);

   char *pszServID = MT_GetServiceKeyByServiceString(pLastService->m_szData);
   if(!pszServID)
   {
	MT_PRINT(0,"CBouquetsReader::LoadServicesFile - Couldn't generate service key. Should never happen.. DOH (Line %i)",uLineNo);
	delete pLastService;
	pLastService = NULL;
   }
 
   if(pLastService)
   {	 
    CService *pOverServ;
    if(m_aServices.Lookup(pszServID,(void*&)pOverServ))
		MT_PRINT(0,"CBouquetsReader::LoadServices - m_aServices.SetAt is overwriting %s with %s",pOverServ->m_szData, pLastService->m_szData);

    m_aServices.SetAt(pszServID,pLastService); 
	delete [] pszServID;
	pszServID = NULL;
   }

  }else if(pLastService)	// Otherwise it's service name or parameters
  {
   SetupServiceFromString(pLastService,szBuff);
  }
 }

 fclose(p);
 
 return true;
}

CService *CBouquetsReader::GetService(const char *pszID)
{
 CService *pServ;

 if(!m_aServices.Lookup(pszID,(void*&)pServ))
	 return NULL;

 return pServ;
}

CService *CBouquetsReader::CreateServiceFromString(const char *pszString)
{
 CService *pServ = new CService();
 if(!pServ)
	 return NULL;

 strcpy(pServ->m_szData,pszString);
 pServ->m_szData[strlen(pszString) - 1] = '\0';

 // -> HACK/FIX: sometimes happens that some settings stores strings differently
 // between service file and bouquet files (lowercase/uppercase). So we just put
 // everything lowercase.
 _strlwr(pServ->m_szData);

 char *pszType = SubStrInSeparatedList(':',5,pszString);
 if(pszType)
 {
  pServ->m_byType = (BYTE)atoi(pszType);
  
  delete [] pszType;
  pszType = NULL;		// Just to be sure..
 }

 return pServ;
}

void CBouquetsReader::SetupServiceFromString(CService *pServ, const char *pszString)
{
 int iStrLen = strlen(pszString);

 // FIXME: you should find more info on services file
 // -> If it's not in the format p:something f:0x0000 g:aaaaaa then its channel name
 // But damn, there is another exception... "2:" is a channel name...
 // C:DIRECT is a channel name too....
 if((pszString[1] != ':' ) || 
    (pszString[1] == ':' && (!isalpha(pszString[0]) || isupper(pszString[0]))))	
 {
  pServ->m_pszName = new char[iStrLen];	  
  strncpy(pServ->m_pszName,pszString,iStrLen);
  pServ->m_pszName[iStrLen - 1] = '\0';
  return;
 };

 /* 
  If string starts with p: then it's defninitly a package name...
  FIXME: What's c: ? 
 */
 if(pServ->m_pszPackage)
	 return;	// package name was already specified... FUCKING EDITORS!

 CParser strParser;
 strParser.Start(pszString,":,\n");

 char *pToken = NULL;
 bool bNextIsProvider = false;
 int iTokenLen = 0;

 do
 {
  pToken = strParser.GetNextToken();
  if(!pToken)
	  continue;

  // -> Check how long newly scanned token is
  iTokenLen = strlen(pToken);
  if(!bNextIsProvider && (iTokenLen == 1))
  {
   bNextIsProvider = true;
   continue;
  }

  // -> We reached a provider name so copy it in our buffer...
  if(bNextIsProvider)
  {
   pServ->m_pszPackage = new char[iTokenLen];	  
   strncpy(pServ->m_pszPackage, pToken, iTokenLen);
   pServ->m_pszPackage[iTokenLen - 1] = '\0'; 
   return;
  }
 }while(pToken);
}
