/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#if !defined(AFX_BOUQUETSREADER_H__86CB754F_D658_4899_8985_1B2F21FDC319__INCLUDED_)
#define AFX_BOUQUETSREADER_H__86CB754F_D658_4899_8985_1B2F21FDC319__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MT_SERVICES_FILE			"services"
#define MT_BOUQUETS_FILE			"bouquets"
#define	MT_TV_USERBOUQUETS_FILE		"userbouquets.tv.epl"
#define	MT_RADIO_USERBOUQUETS_FILE	"userbouquets.radio.epl"

#define MT_SRVID_IN_SERVICE_FILE	1
#define MT_NMSID_IN_SERVICE_FILE	2
#define MT_TRNID_IN_SERVICE_FILE	3
#define MT_NETID_IN_SERVICE_FILE	4

#define MT_SRVID_IN_BOUQUET_FILE	4
#define MT_TRNID_IN_BOUQUET_FILE	5


class CBouquetNode {
public:
	CBouquetNode()
	{
	 m_aChannels.RemoveAll();
	 m_aBouquets.RemoveAll();

	 m_szName[0] = '\0';
	 m_szPath[0] = '\0';
	};

	~CBouquetNode()
	{
     int i = 0;
     for(i; i < m_aChannels.GetSize(); ++i)
	 {
	  CBouquetNode *pNode = (CBouquetNode*)m_aChannels.GetAt(i);
	  if(!pNode)
		  continue;

	  delete pNode;

	  m_aChannels.SetAt(i,NULL);
	 }

     for(i = 0; i < m_aBouquets.GetSize(); ++i)
	 {
	  CBouquetNode *pNode = (CBouquetNode*)m_aBouquets.GetAt(i);
	  if(!pNode)
		  continue;

	  delete pNode;

	  m_aBouquets.SetAt(i,NULL);
	 }
	};

	char	m_szName[128];			// Service Name
	char	m_szPath[255];			// If that's a service, then that's service string. Otherwise it is path to boquets file

	CPtrArray	m_aChannels;		// Has sub items
	CPtrArray   m_aBouquets;		// Has items on same level
};

class CService {
public:
	CService()
	{
	 m_pszName = NULL;
	 m_pszPackage = NULL;
	};

	~CService()
	{
	 if(m_pszName)
		 delete [] m_pszName;

	 if(m_pszPackage)
		 delete [] m_pszPackage;

	 m_pszName = NULL;
	 m_pszPackage = NULL;
	};

	/*
		m_szData is a string like 1:0:1:d49:1450:13e:820000:0:0:0: where :

		"d49"		: Service ID
		"820000"	: Namespace 
	*/

	char	m_szData[64];			// (#SERVICE:) 1:0:1:d49:1450:13e:820000:0:0:0:
	BYTE	m_byType;				// 1 TV; 2 RADIO; Other Data

	char	*m_pszName;				// ie : "Game Network"
	char	*m_pszPackage;			// ie : "Telespazio"
};

class CBouquetsReader  
{
public:
	CBouquetsReader();
	~CBouquetsReader();

	void	Clear();
	void	Destroy();
	bool	Init();

	bool	Open();
	void	Close();
	bool	GetMasterFile(const char*);

	bool	ScanTVMasterFile();
	bool	ScanRadioMasterFile();
	bool	ScanBouquetsMasterFile();
	bool	LoadServicesFile();

	CService *GetService(const char *pszID);

	CBouquetNode		m_RootTVNode;
	CBouquetNode		m_RootRadioNode;
	CBouquetNode		m_BouquetNode;

private:
	CFtpConnection		*m_pFTP;
	CMapStringToPtr		m_aServices;

	int		ParseBouquetsFile(CBouquetNode *pRootNode,const char *pszFileName);		// Returns number of found paths
	int		ParseUserBouquetsFile(CBouquetNode *pRootNode,const char *pszFileName);	// Returns number of found paths
	CService	*CreateServiceFromString(const char *pszString);
	void	SetupServiceFromString(CService *pServ, const char *pszString);
};

#endif // !defined(AFX_BOUQUETSREADER_H__86CB754F_D658_4899_8985_1B2F21FDC319__INCLUDED_)
