/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#include "stdafx.h"
#include "MTuxvision.h"
#include "MTBouquetsView.h"
#include "MT_Utils.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CTreeCtrl *MT_GetBouquetsCtrl()
{
 CMTBouquetsView *pView = (CMTBouquetsView *)((CMainFrame*)MT_GetApp()->m_pMainWnd)->m_wndSplitter.GetPane(0,3);
 if(!pView)
	 return NULL;

 return &pView->GetTreeCtrl();
}

/////////////////////////////////////////////////////////////////////////////
// CMTBouquetsView

IMPLEMENT_DYNCREATE(CMTBouquetsView, CTreeView)

CMTBouquetsView::CMTBouquetsView()
{
 m_dwDefaultStyle |= (TVS_HASLINES | TVS_SHOWSELALWAYS | TVS_LINESATROOT | TVS_HASBUTTONS);
}

CMTBouquetsView::~CMTBouquetsView()
{
 ((CMainFrame*)MT_GetApp()->GetMainWnd())->SetActiveView(NULL);
}


BEGIN_MESSAGE_MAP(CMTBouquetsView, CTreeView)
	//{{AFX_MSG_MAP(CMTBouquetsView)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelchanged)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMTBouquetsView drawing

void CMTBouquetsView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CMTBouquetsView diagnostics

#ifdef _DEBUG
void CMTBouquetsView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CMTBouquetsView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMTBouquetsView message handlers

void CMTBouquetsView::OnInitialUpdate() 
{
	CTreeView::OnInitialUpdate();

	GetTreeCtrl().SetBkColor(MTUXVISION_SCHEDULE_BG);
	
	LoadBouquets();
}

void CMTBouquetsView::Traverse(HTREEITEM hParent, CBouquetNode *pParent)
{
 CTreeCtrl *pTree = &GetTreeCtrl();
 int i;

 for(i = 0; i <= pParent->m_aBouquets.GetUpperBound(); ++i)
 {
  CBouquetNode *pNode = (CBouquetNode*)pParent->m_aBouquets.GetAt(i);
  if(!pNode)
	  continue;

  HTREEITEM hNew = pTree->InsertItem(pNode->m_szName,hParent);
  pTree->SetItemData(hNew,(DWORD)pNode);

  Traverse(hNew,pNode);
 }
};

void CMTBouquetsView::TraverseChan(HTREEITEM hParent, CBouquetNode *pParent)
{
 CListCtrl *pList = MT_GetChanCtrl();

 int iNum = pParent->m_aChannels.GetUpperBound();

 for(int i = 0; i <= iNum; ++i)
 {
  CBouquetNode *pNode = (CBouquetNode*)pParent->m_aChannels.GetAt(i);
  if(!pNode)
	  continue;

  pList->InsertItem(i,pNode->m_szName);
  pList->SetItemData(i,(DWORD)pNode);

  char *pszSrvID = MT_GetServiceKeyByBouquetString(pNode->m_szPath);
  if(!pszSrvID)
  {
	  // -> Let's give it another chance..
	  pszSrvID = MT_GetServiceKeyByServiceString(pNode->m_szPath);
	  if(!pszSrvID)
		  continue;
  }

  StrTrimLeft('0',pszSrvID);

  // -> Let's find out if that's TV , Radio or Data
  CService *pServ = MT_GetServiceByID(pszSrvID);
  if(!pServ)
  {
	  delete [] pszSrvID;
	  continue;
  }

  switch(pServ->m_byType)
  {
  case 1:
   pList->SetItemText(i,1,"TV");
   break;
  case 2:
   pList->SetItemText(i,1,"Radio");
   break;
  default:
   pList->SetItemText(i,1,"Data");
   break;
  }
 }
}

void CMTBouquetsView::OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	HTREEITEM hNew = GetTreeCtrl().GetSelectedItem();
	if(hNew != NULL)
	{
	 MT_GetChanCtrl()->DeleteAllItems();

	 CBouquetNode *pNode = (CBouquetNode*)(DWORD)GetTreeCtrl().GetItemData(hNew);
	 if(pNode)
	 {
	  TraverseChan(hNew,pNode);
	 }
	}

	*pResult = 0;
}

bool CMTBouquetsView::LoadBouquets()
{
	char szTemp[255];

	// -- Gener
	m_hUserTV		= GetTreeCtrl().InsertItem("TV User Bouquets");
	m_hUserRadio	= GetTreeCtrl().InsertItem("Radio User Bouquets");
	m_hBouquets		= GetTreeCtrl().InsertItem("Bouquets");

	CMTuxvisionApp *pApp = MT_GetApp();
	if(!pApp)
	{
     MT_PRINT(0,"CMTBouquetsView::LoadBouquets - Failed to get a pointer to the application");
	 return false;
	}

	// -> This will surely exist
	CBouquetsReader *pBouquets = &pApp->m_cBouquetsReader;

	MT_PRINT(0,"Populating bouquets view control...");

	if(!pBouquets->Open())
	{
	 MT_PRINT(0,"CMTBouquetsView::LoadBouquets - Failed to connect to dreambox. Loading cached files...");
	}

	/*
	   *** SERVICES ***
	*/
    sprintf(szTemp,"%s/%s",g_sGlobals.m_szBouquetPath, MT_SERVICES_FILE);
    if(pBouquets->GetMasterFile(szTemp))
	{
     if(!pBouquets->LoadServicesFile())
	 {
      MT_PRINT(0,"CMTBouquetsView::LoadBouquets - Unable to load services. Fatal error. Sending quit message...");
  
      pBouquets->Close();
      PostQuitMessage(0);
      return false;
	 };
	};

	/*
	   *** TV USER BOUQUETS ***
	*/
	sprintf(szTemp,"%s/%s",g_sGlobals.m_szBouquetPath, MT_TV_USERBOUQUETS_FILE);

	if(pBouquets->GetMasterFile(szTemp))
	{
	 if(!pBouquets->ScanTVMasterFile())
	 {
	  MT_PRINT(0,"CMTBouquetsView::LoadBouquets - Unable to scan master tv file. Fatal error. Sending quit message...");
	  
	  pBouquets->Close();
	  PostQuitMessage(0);
	  return false;
	 };
	};

	/*
	   *** RADIO USER BOUQUETS ***
	*/
	sprintf(szTemp,"%s/%s",g_sGlobals.m_szBouquetPath, MT_RADIO_USERBOUQUETS_FILE);

	if(pBouquets->GetMasterFile(szTemp))
	{
	 if(!pBouquets->ScanRadioMasterFile())
	 {
	  MT_PRINT(0,"CMTBouquetsView::LoadBouquets - Unable to scan master radio file. Fatal error. Sending quit message...");
	  
	  pBouquets->Close();
	  PostQuitMessage(0);
	  return false;
	 };
	};

	/*
	   *** GENERAL TV BOUQUETS ***
	*/
	sprintf(szTemp,"%s/%s",g_sGlobals.m_szBouquetPath, MT_BOUQUETS_FILE);	
	if(pBouquets->GetMasterFile(szTemp))
	{
	 pBouquets->ScanBouquetsMasterFile();
	};
		
	pBouquets->Close();

	/*
	   *** FINALLY ADD RESULTS TO THE LIST ***
	*/
	Traverse(m_hUserTV,&pBouquets->m_RootTVNode);
	Traverse(m_hUserRadio, &pBouquets->m_RootRadioNode);
	Traverse(m_hBouquets,&pBouquets->m_BouquetNode);

	return true;
}
