/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#include "stdafx.h"
#include "MTuxvision.h"
#include "MainFrm.h"
#include "MTuxvisionDoc.h"
#include "MTChannelView.h"
#include "MTuxvisionView.h"
#include "Engine/DBoxComm.h"
#include "CustomPID.h"
#include "gui/syncdialog.h"

CSyncDialog g_Sync;

CListCtrl *MT_GetChanCtrl()
{
 CMTChannelView *pView = (CMTChannelView *)((CMainFrame*)MT_GetApp()->m_pMainWnd)->m_wndSplitter.GetPane(0,2);
 if(!pView)
	 return NULL;

 return &pView->GetListCtrl();
}

UINT StartSyncDialog(LPVOID pParam)
{
 g_Sync.DoModal();

 return 0;
}

void StopSyncDialog()
{
 g_Sync.EndDialog(0);
}

/////////////////////////////////////////////////////////////////////////////
// CMTChannelView

IMPLEMENT_DYNCREATE(CMTChannelView, CListView)

CMTChannelView::CMTChannelView()
{
 m_dwDefaultStyle |= (LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL );
}

CMTChannelView::~CMTChannelView()
{
}


BEGIN_MESSAGE_MAP(CMTChannelView, CListView)
	//{{AFX_MSG_MAP(CMTChannelView)
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnItemchanged)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	ON_COMMAND(ID_MAIN_SETAUDIOPID, OnMainSetaudiopid)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMTChannelView drawing

void CMTChannelView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CMTChannelView diagnostics

#ifdef _DEBUG
void CMTChannelView::AssertValid() const
{
	CListView::AssertValid();
}

void CMTChannelView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMTChannelView message handlers

void CMTChannelView::OnInitialUpdate() 
{
	CListView::OnInitialUpdate();

	GetListCtrl().SetBkColor(MTUXVISION_SCHEDULE_BG);
	GetListCtrl().SetTextBkColor(MTUXVISION_SCHEDULE_BG);

	ListView_SetExtendedListViewStyleEx(GetSafeHwnd(),0,LVS_EX_FULLROWSELECT);
	
	GetListCtrl().DeleteAllItems();

	// -> Setup columns
	GetListCtrl().InsertColumn(0,"Service",LVCFMT_LEFT,135);
	GetListCtrl().InsertColumn(1,"Type",LVCFMT_LEFT,40);	
}

void CMTChannelView::OnDestroy() 
{
	GetListCtrl().DeleteAllItems();
	
	CListView::OnDestroy();	
}

void CMTChannelView::OnSize(UINT nType, int cx, int cy) 
{
	CListView::OnSize(nType, cx, cy);
	
	if(IsWindow(GetListCtrl().GetSafeHwnd()))
	{
	 GetListCtrl().SetColumnWidth(0, cx - 40);
	 GetListCtrl().SetColumnWidth(1, 40);
	}
}

void CMTChannelView::OnItemchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	if(pNMListView->uNewState & LVIS_SELECTED)
	{
     CBouquetNode *pNode = (CBouquetNode*)GetListCtrl().GetItemData(pNMListView->iItem);
	 if(pNode)
	 {
	  AfxBeginThread(StartSyncDialog,NULL);
	  
	  if(g_DBoxComm.SetChannel(pNode->m_szPath))
	  {
	   strncpy(g_sGlobals.m_szCurChanName,pNode->m_szName,32);

	   MT_GetApp()->GetChannelInfo();

	   if(g_sGlobals.m_bRecAndPreview)
			MT_GetApp()->StartPreview();

	   CMTuxvisionView *pView = MT_GetVideoView();
	   if(pView)
		   pView->InvalidateRect(NULL,FALSE);

	  }

	  char szTemp[255];
      sprintf(szTemp,"%s (%s)",pNode->m_szName,g_sGlobals.m_szFormat);

	  ((CMainFrame*)MT_GetApp()->m_pMainWnd)->m_wndStatusBar.SetPaneText(0,szTemp);
	  g_Sync.EndDialog(0);
	 }
	}
	
	*pResult = 0;
}

void CMTChannelView::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CMenu menuPopup;
	NMLISTVIEW *pListItem = (NMLISTVIEW*)pNMHDR;

	// -> Nothing selected, exit...
	if(pListItem->iItem == -1)
	{
		*pResult = 0;
		return;
	}

	if(menuPopup.LoadMenu(MAKEINTRESOURCE(IDR_CHANNEL_MENU)))
	{
	 CPoint pt;
	 GetCursorPos(&pt);

	 menuPopup.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN,pt.x,pt.y,this);

	 menuPopup.DestroyMenu();
	}

	*pResult = 0;
}

void CMTChannelView::OnMainSetaudiopid() 
{
 CCustomPid pidDlg;
	
 if(pidDlg.DoModal() == IDOK)
 {

 }
}
