/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#include "stdafx.h"
#include "MT_Utils.h"
#include "Engine\BouquetsReader.h"
#include "Parser.h"

#include <shlobj.h>

char * StripFileName(const char* szPath)
{
 char *pFinal = strrchr(szPath,'/');
 if(pFinal)
 {
  return ++pFinal; 
 }

 return (char*)szPath;	// Return starting buffer...
}

int CountChrInStr(char c, const char *pszString,int iExitAtCnt)
{
 int iLen = strlen(pszString), iCnt = 0;

 for(int k = 0; k < iLen; ++k)
 {
  if(pszString[k] == c)
  {
   iCnt++;

   if(iCnt >= iExitAtCnt)
	   return iCnt;
  }
 }

 return iCnt;
}

char * SubStrInSeparatedList(char cSeparator,int iSubStringNumber,const char *pszString)
{
 int iLen = strlen(pszString);
 int iIndex = 0,iStr = 0;
 char *pszNew = new char[25];

 for(int k = 0; k < iLen; ++k)
 {
  if(pszString[k] == cSeparator)
  {
	  iIndex++;

	  if(iIndex == iSubStringNumber)
	  {
		  pszNew[iStr] = '\0';
		  return pszNew;
	  }

	  iStr = 0;
	  continue;
  }

  pszNew[iStr++] = pszString[k];
 }

 delete [] pszNew;

 return NULL;
}

char * StrTrimLeft(char c, char *pszString)
{
 int iLen = strlen(pszString);
 int iCharsToRemove = 0;
 int k = 0;

 for(k; k < iLen; ++k)
 {
  if(pszString[k] != c)
	  break;

  ++iCharsToRemove;
 }

 if(!iCharsToRemove)
 {
  // ? What now ?
  return NULL;
 }

 int iIndex = 0;

 for(k; k < iLen; ++k)
 {
  pszString[iIndex++] = pszString[k];
 }

 pszString[iIndex] = '\0';

 return pszString;
}

char * StrTrimLeftAlloc(char c, const char *pszString)
{
 int iLen = strlen(pszString);
 int iCharsToRemove = 0;
 int k = 0;

 for(k; k < iLen; ++k)
 {
  if(pszString[k] != c)
	  break;

  ++iCharsToRemove;
 }

 if(!iCharsToRemove)
 {
  // ? What now ?
  return NULL;
 }

 char *pszNew = new char[iLen - iCharsToRemove];
 if(!pszNew)
	 return NULL;
 
 int iIndex = 0;

 for(k; k < iLen; ++k)
 {
  pszNew[iIndex++] = pszString[k];
 }

 pszNew[iIndex] = '\0';

 return pszNew;
}

char * MT_GetServiceKeyByServiceString(const char *pszString)
{
 char *pServiceID = SubStrInSeparatedList(':',MT_SRVID_IN_SERVICE_FILE,pszString);
 if(!pServiceID)
	 return NULL;

 char *pTranspoID = SubStrInSeparatedList(':',MT_TRNID_IN_SERVICE_FILE,pszString);
 if(!pTranspoID)
	 return NULL;

 char *pRet = new char[strlen(pServiceID) + strlen(pTranspoID) + 2];
 if(!pRet)
 {
  delete [] pServiceID;
  delete [] pTranspoID;
  return NULL;
 }

 StrTrimLeft('0',pServiceID);
 StrTrimLeft('0',pTranspoID);

 strcpy(pRet,pServiceID);
 strcat(pRet,":");
 strcat(pRet,pTranspoID);

 delete [] pServiceID;
 delete [] pTranspoID;

 return pRet;
}

char * MT_GetServiceKeyByBouquetString(const char *pszString)
{// REWRITE ME! THIS COULD BE OPTIMIZED ASS!
 char *pServiceID = SubStrInSeparatedList(':',MT_SRVID_IN_BOUQUET_FILE,pszString);
 if(!pServiceID)
	 return NULL;

 char *pTranspoID = SubStrInSeparatedList(':',MT_TRNID_IN_BOUQUET_FILE,pszString);
 if(!pTranspoID)
	 return NULL;

 char *pRet = new char[strlen(pServiceID) + strlen(pTranspoID) + 2];
 if(!pRet)
 {
  delete [] pServiceID;
  delete [] pTranspoID;
  return NULL;
 }

 StrTrimLeft('0',pServiceID);
 StrTrimLeft('0',pTranspoID);

 strcpy(pRet,pServiceID);
 strcat(pRet,":");
 strcat(pRet,pTranspoID);

 delete [] pServiceID;
 delete [] pTranspoID;

 return pRet;
}

/*
 Takes : 0002:00820000:1770:0110:1
 Returns : 1:0:1:2:1770:110:820000:0:0:0:
*/
char * MT_BuildBouquetStringByServiceString(const char *pszString)
{
 char *pServiceID = SubStrInSeparatedList(':',MT_SRVID_IN_SERVICE_FILE,pszString);
 if(!pServiceID)
	 return NULL;

 char *pTranspoID = SubStrInSeparatedList(':',MT_TRNID_IN_SERVICE_FILE,pszString);
 if(!pTranspoID)
 {
  delete [] pServiceID;

  return NULL;
 }

 char *pNamespaID = SubStrInSeparatedList(':',MT_NMSID_IN_SERVICE_FILE,pszString);
 if(!pNamespaID)
 {
  delete [] pServiceID;
  delete [] pTranspoID;

  return NULL;
 }

 char *pNetworkID = SubStrInSeparatedList(':',MT_NETID_IN_SERVICE_FILE,pszString);
 if(!pNetworkID)
 {
  delete [] pServiceID;
  delete [] pTranspoID;
  delete [] pNamespaID;

  return NULL;
 }

 char *pNew = new char[17 + strlen(pServiceID) + strlen(pTranspoID) + strlen(pNamespaID) + strlen(pNetworkID)];
 if(!pNew)
 {
  delete [] pServiceID;
  delete [] pTranspoID;
  delete [] pNamespaID;
  delete [] pNetworkID;

  return NULL;
 }

 StrTrimLeft('0',pServiceID);
 StrTrimLeft('0',pTranspoID);
 StrTrimLeft('0',pNamespaID);
 StrTrimLeft('0',pNetworkID);

 sprintf(pNew,"1:0:1:%s:%s:%s:%s:0:0:0:",pServiceID,pTranspoID,pNetworkID,pNamespaID);

 return pNew;
}

char * MT_ShowFolderDialog()
{
 BROWSEINFO bi;
 LPITEMIDLIST pidl;
 LPMALLOC pMalloc;

 if(SUCCEEDED(SHGetMalloc(&pMalloc))) 
 {
  ZeroMemory(&bi,sizeof(bi));
  bi.hwndOwner = NULL;
  bi.pszDisplayName = 0;
  bi.pidlRoot = 0;
  bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;
  bi.lpszTitle = "Select folder to output video to:\0"; 
//  bi.lpfn = BrowseCallbackProc;

  pidl = SHBrowseForFolder(&bi);
  if(pidl) 
  {
   char * pszDir = new char[MAX_PATH];
   if(!pszDir)
   {
    pMalloc->Free(pidl);
    pMalloc->Release();
	return NULL;
   }

   if (SHGetPathFromIDList(pidl,pszDir)) 
   {
    MessageBox(NULL,pszDir,"Picked",MB_OK);
   }

   pMalloc->Free(pidl);
   pMalloc->Release();

   return pszDir;
  }
 }


 return NULL;
}

char * MT_GenerateMovieFileName(const char *pszChanTitle)
{
 char * pNewStr = new char[255];
 if(!pNewStr)
	 return NULL;

 SYSTEMTIME sysTime;

 GetLocalTime(&sysTime);

 if(pszChanTitle)
	sprintf(pNewStr,"%s_%i%i%i_%i%i",pszChanTitle,sysTime.wDay,sysTime.wMonth,sysTime.wYear,sysTime.wHour,sysTime.wMinute);
 else
	sprintf(pNewStr,"mtux_%i%i%i_%i%i",sysTime.wDay,sysTime.wMonth,sysTime.wYear,sysTime.wHour,sysTime.wMinute);
  
 return pNewStr;
}

static char gszSeps[] = "\n\t\"<>:/ ";
char *MT_GetValueInHTMLTag(char *pszHTML,const char *pValue,int iLen)
{
 bool bSkip = true;

 if(iLen < 0)
	g_strParser.Start(pszHTML,gszSeps);
 else
	g_strParser.Start(pszHTML,iLen,gszSeps);

 char *pToken = g_strParser.GetNextToken();
 
 do
 {
  pToken = g_strParser.GetNextToken();
  if(!pToken)
	  return NULL;

  if(strcmpi(pToken,pValue) == 0)	
  {
   bSkip = false;
   continue;
  }

  if(bSkip)
	  continue;

  if((strcmpi(pToken,"td") == 0) || (strcmpi(pToken,"tr") == 0))
	  continue;

  return pToken;

 }while(pToken);

 return NULL;
}

void MT_AddPIDsToList(char *pszHTML,CWnd *pWnd)
{
 CComboBox *pCombo = (CComboBox*)pWnd;
 if(!pWnd || !pszHTML)
	 return;

 int iLen = strlen(pszHTML);

 g_strParser.Start(pszHTML, iLen, gszSeps);

 char *pToken = NULL;

 do
 {
  pToken = g_strParser.GetNextToken();
  if(!pToken)
	  return;

  if(strcmpi(pToken,"option") == 0)
	  pCombo->AddString("option");

 }while(pToken);
}
