/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#ifndef _MTUXVISION_SHARED_H_
#define _MTUXVISION_SHARED_H_

class CLogger;
class CMTuxvisionApp;
class CListCtrl;
class CTreeCtrl;
class CBouquetsReader;
class CService;
class CVideoPrevWnd;
class CInternetSession;
class CMTuxvisionView;

// --- Custom windows messages ---
#define WM_STREAMNOTIFY	(WM_USER + 4000)


// --- Function Headers ---
CMTuxvisionApp		*MT_GetApp();
CListCtrl			*MT_GetChanCtrl();
CTreeCtrl			*MT_GetBouquetsCtrl();
CService			*MT_GetServiceByID(const char *pszID);
CVideoPrevWnd		*MT_GetVideoPrevWnd();
CMTuxvisionView		*MT_GetVideoView();

// --- From MT_Channeview.h ---
UINT StartSyncDialog(LPVOID pParam);
void StopSyncDialog();

// --- From Tuxvision.h ---
typedef enum { 
	StateUninitialized,
	StateStopped, 
	StatePreview, 
	StateRecord, 
	StatePlayback 
} RecorderState;

#define AUDIO_PCM       0
#define AUDIO_MPEG1L2   1
#define AUDIO_MPEG1L3   2

#define pWidth(x)   (x->right-x->left)
#define pHeight(x)  (x->bottom-x->top)

// --- MTuxvision ---
#define MTUXVISION_VERSION	0.2
#define MTUXVISION_BUILD	15
#define MTUXVISION_KEY_NAME "Software\\MTuxvision"
#define MTUXVISION_SCHEDULE_BG		RGB(185,253,197)

typedef enum {
	Format16by9,
	Format4by3
} VideoFormat;

typedef enum {
	EpgSpallared,
	EpgMultiview,
	EpgRaw
} EpgFormat;

typedef struct {
	/* Dreambox Status */
	int				m_iDreamboxOnline;		// Is Dreambox online? (FTP check)
	VideoFormat		m_eVideoFormat;			// 16:9 or 4:3
	RecorderState	m_dbState;				// What is dreambox doing atm?
	char			m_szCurChanName[32];	// Current selected channel name
	char			m_szAPid[10];			// Current audio PID
	char			m_szVPid[10];			// Current video PID
	char			m_szFormat[15];			// 740x330... (ie)
	char			m_szLanguage[15];		// italian (.xml is automatically appended)


	/* Dreambox Communication Options */
	bool			m_bPassiveFTP;			// Ftp in passive mode or not
	char			m_szUsername[25];		// FTP username
	char			m_szPassword[25];		// FTP password
	BYTE			m_byDreamboxIP[4];		// Dreambox IP
	INTERNET_PORT	m_portHttp;				// Dreambox HTTP port
	INTERNET_PORT   m_portFtp;				// Dreambox FTP port
	char			m_szBouquetPath[255];	// Bouquets path

	/* Application Common Options */
	bool			m_bFullscreen;			// Is fullscreen preview enabled?
	bool			m_bAlwaysOnTop;			// Top most window?
	bool			m_bRecAndPreview;		// Preview while recording
	bool			m_bUseDeinterlacer;		// Deinterlace stream
	EpgFormat		m_eEpgFormat;			// Epg reading method
	BYTE			m_byVolume;				// Volume 0-100
	char			m_szDefaultVideoPath[MAX_PATH];	// Default path for recorded video
}MTGlobals;

// --- Globals ---
extern CLogger g_Logger;
extern CMTuxvisionApp theApp;
extern CInternetSession g_isDreambox;

extern MTGlobals g_sGlobals;

#define MT_PRINT g_Logger.Print

#endif //_MTUXVISION_SHARED_H_
