/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/

#include "stdafx.h"
#include "MTuxvision.h"

#include "caboutdlg.h"

#include "MainFrm.h"
#include "MTuxvisionDoc.h"
#include "MTuxvisionView.h"
#include "MTBouquetsView.h"
#include "gui/splashdialog.h"
/*
#include <dshow.h>
#include <atlbase.h>
*/

#include <streams.h>

#include <initguid.h>

#include "engine\DShow.h"

#include "engine\guids.h"
#include "..\render\interface.h"
#include "..\audiofilerenderer\interface.h"

#include "Engine\DBoxComm.h"
#include "Engine\AX_Helper.h"

#include "MT_Utils.h"
#include "singleinstance.h"
#include "parser.h"

/* --- SOME GLOBALS --- */
CLimitSingleInstance g_SingleInstanceObj(TEXT("{1FE562DA-5B2F-4043-89DE-1324AB614CA5}"));

CLogger g_Logger;
CInternetSession g_isDreambox("MTuxvision");

MTGlobals	g_sGlobals;

//CSplashDialog *g_pSplash = NULL;		// Splash screen

/* --- END GLOBALS SECTION --- */
/*
UINT MT_SplashThread( LPVOID pParam )
{
 g_pSplash->DoModal();

 return 0;
}*/

void PrintLogHeader()
{
 MT_PRINT(0,"--- MTuxvision V %0.1f Build %i ---", MTUXVISION_VERSION, MTUXVISION_BUILD);
 MT_PRINT(0,"");
}

/////////////////////////////////////////////////////////////////////////////
// CMTuxvisionApp

BEGIN_MESSAGE_MAP(CMTuxvisionApp, CWinApp)
	//{{AFX_MSG_MAP(CMTuxvisionApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMTuxvisionApp construction

CMTuxvisionApp::CMTuxvisionApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMTuxvisionApp object

CMTuxvisionApp theApp;

CMTuxvisionApp *MT_GetApp()
{
 return &theApp;
};

/////////////////////////////////////////////////////////////////////////////
// CMTuxvisionApp initialization

BOOL CMTuxvisionApp::InitInstance()
{
	if(g_SingleInstanceObj.IsAnotherInstanceRunning())
	{
	 AfxMessageBox("You cannot run MTuxvision twice!");
	 return false;
	};

	// Standard initialization
	CoInitialize(NULL);

	if(!GetAppPath())
	{
	 MessageBox(NULL,"Unable to find MTuxvision path in registry. Corrupted installation","Fatal Error",MB_OK);
	 return false;
	}

	ILogFile *pLogInterface = new ILogFile();
	if(pLogInterface)
	{
	 char *pFileName = new char[255];
	 sprintf(pFileName,"%s/mtuxvision.txt",m_szAppPath);

     pLogInterface->Open(pFileName);
	 g_Logger.SetMediaInterface(pLogInterface);

	 delete [] pFileName;
	}

	PrintLogHeader();

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	SetRegistryKey(_T("MTuxvision"));

	// -> Load and show splash screen while loading
/*	g_pSplash = new CSplashDialog();
	if(!g_pSplash)
		return FALSE;

	AfxBeginThread(MT_SplashThread,NULL);*/

	LoadGlobals();
	
	MT_PRINT(0,"Initializing bouquet analyzer...");

	if(!m_cBouquetsReader.Init())	 
	{
		MT_PRINT(0,"MTuxvision was unable to initialize bouquets reader...");
		return false;
	}

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CMTuxvisionDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		NULL);
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// -> This one needs GUI already initialized and working!
	InitInterfaces();

	// The one and only window has been initialized, so show and update it.
	m_pMainWnd->SetWindowPos(g_sGlobals.m_bAlwaysOnTop ? &CWnd::wndTopMost : &CWnd::wndNoTopMost,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	/*if(g_pSplash)
	{
	 g_pSplash->EndDialog(0);
	 delete g_pSplash;
	 g_pSplash = NULL;
	}*/

	char szTemp[MAX_PATH];
	sprintf(szTemp, "%s\\lang\\%s.xml", m_szAppPath, g_sGlobals.m_szLanguage);
	
	if(!LoadLanguage(szTemp,false))
		MT_PRINT(0,"CMTuxvisionApp::InitInstance - Unable to load language file");

	return TRUE;
}

void CMTuxvisionApp::LoadGlobals()
{
 MT_PRINT(3,"Loading globals from registry...");

 // -> FIXME : make me bullet proof

 g_sGlobals.m_iDreamboxOnline = 1;
 g_sGlobals.m_dbState		  = StateUninitialized;
 g_sGlobals.m_bFullscreen	  = false;
 g_sGlobals.m_bRecAndPreview  = true;	
 strcpy(g_sGlobals.m_szAPid,"0000h");
 strcpy(g_sGlobals.m_szVPid,"0000h");
 strcpy(g_sGlobals.m_szFormat,"unknown");

 strcpy(g_sGlobals.m_szCurChanName,"error");
 strcpy(g_sGlobals.m_szLanguage,"default");

 CRegKey reg;

 if(reg.Open(HKEY_LOCAL_MACHINE,MTUXVISION_KEY_NAME) == ERROR_SUCCESS)
 {
  DWORD dwTemp = 25;
  char szTemp[25];

  reg.QueryValue(g_sGlobals.m_szUsername,"Username",&dwTemp);

  dwTemp = 25;
  reg.QueryValue(g_sGlobals.m_szPassword,"Password",&dwTemp);

  reg.QueryValue(dwTemp,"DreamboxIP");
  g_sGlobals.m_byDreamboxIP[0] = (BYTE)(dwTemp >> 24);
  g_sGlobals.m_byDreamboxIP[1] = (BYTE)(dwTemp >> 16);
  g_sGlobals.m_byDreamboxIP[2] = (BYTE)(dwTemp >> 8);
  g_sGlobals.m_byDreamboxIP[3] = (BYTE)dwTemp;

  reg.QueryValue(dwTemp,"HTTPport");
  g_sGlobals.m_portHttp = (INTERNET_PORT)dwTemp;

  reg.QueryValue(dwTemp,"FTPport");
  g_sGlobals.m_portFtp  = (INTERNET_PORT)dwTemp;

  dwTemp = 25;
  reg.QueryValue(szTemp,"UseDeinterlacer",&dwTemp);
  g_sGlobals.m_bUseDeinterlacer = (atoi(szTemp) == 0) ? false : true;

  dwTemp = 25;
  reg.QueryValue(szTemp,"Format16by9",&dwTemp);
  g_sGlobals.m_eVideoFormat = (atoi(szTemp)) ? Format16by9 : Format4by3;

  dwTemp = MAX_PATH;
  reg.QueryValue(g_sGlobals.m_szDefaultVideoPath,"DefaultVideoPath",&dwTemp);

  dwTemp = 25;
  reg.QueryValue(szTemp,"PassiveFTP",&dwTemp);
  g_sGlobals.m_bPassiveFTP = (atoi(szTemp)) ? true : false;

  dwTemp = 25;
  reg.QueryValue(szTemp,"Topmost",&dwTemp);
  g_sGlobals.m_bAlwaysOnTop = (atoi(szTemp)) ? true : false;

  reg.QueryValue(dwTemp,"Volume");
  g_sGlobals.m_byVolume = (BYTE)dwTemp;

  dwTemp = 15;
  reg.QueryValue(g_sGlobals.m_szLanguage,"Language",&dwTemp);

  dwTemp = 255;
  reg.QueryValue(g_sGlobals.m_szBouquetPath, "RemotePath", &dwTemp);

  reg.Close();
  return;
 }

 MT_PRINT(0,"CMTuxvisionApp::LoadGlobals - Unable to load settings!");

 strcpy(g_sGlobals.m_szUsername,"root");
 strcpy(g_sGlobals.m_szPassword,"dreambox");
 
 g_sGlobals.m_byDreamboxIP[0] = 192;
 g_sGlobals.m_byDreamboxIP[1] = 168;
 g_sGlobals.m_byDreamboxIP[2] = 0;
 g_sGlobals.m_byDreamboxIP[3] = 24;

 g_sGlobals.m_portHttp = 80;
 g_sGlobals.m_portFtp  = 21;
 g_sGlobals.m_bPassiveFTP = false;

 g_sGlobals.m_eVideoFormat	   = Format4by3;
 g_sGlobals.m_bUseDeinterlacer = false;

 g_sGlobals.m_bAlwaysOnTop	   = false;
 strcpy(g_sGlobals.m_szDefaultVideoPath,"c:");
 strcpy(g_sGlobals.m_szLanguage, "english");
 strcpy(g_sGlobals.m_szBouquetPath, "/var/tuxbox/config/enigma");
}

bool CMTuxvisionApp::SaveGlobals()
{
 MT_PRINT(3,"Saving globals to registry...");

 CRegKey reg;

 if(reg.Open(HKEY_LOCAL_MACHINE,MTUXVISION_KEY_NAME) != ERROR_SUCCESS)
	return false;

 char szTemp[25];

 reg.SetValue(g_sGlobals.m_szUsername,"Username");
 reg.SetValue(g_sGlobals.m_szPassword,"Password");

 DWORD dwIP = (g_sGlobals.m_byDreamboxIP[0] << 24) | (g_sGlobals.m_byDreamboxIP[1] << 16) |
			  (g_sGlobals.m_byDreamboxIP[2] << 8) | (g_sGlobals.m_byDreamboxIP[3]);

 reg.SetValue(dwIP,"DreamboxIP");
 reg.SetValue(g_sGlobals.m_portHttp,"HTTPport");
 reg.SetValue(g_sGlobals.m_portFtp,"FTPport");
 reg.SetValue(itoa(g_sGlobals.m_bUseDeinterlacer,szTemp,10),"UseDeinterlacer");
 reg.SetValue(itoa(g_sGlobals.m_eVideoFormat,szTemp,10),"Format16by9");
 reg.SetValue(g_sGlobals.m_szDefaultVideoPath,"DefaultVideoPath");

 reg.SetValue(itoa(g_sGlobals.m_bPassiveFTP,szTemp,10),"PassiveFTP");
 reg.SetValue(itoa(g_sGlobals.m_bAlwaysOnTop,szTemp,10),"Topmost");

 reg.SetValue(g_sGlobals.m_byVolume,"Volume");
 reg.SetValue(g_sGlobals.m_szLanguage, "Language");
 reg.SetValue(g_sGlobals.m_szBouquetPath, "RemotePath");

 reg.Close();

 return true;
}

bool CMTuxvisionApp::GetAppPath()
{
 CRegKey reg;

 if(reg.Open(HKEY_LOCAL_MACHINE,MTUXVISION_KEY_NAME) == ERROR_SUCCESS)
 {
  DWORD dwTemp = 255;

  reg.QueryValue(m_szAppPath,"Path",&dwTemp);	   
  
  reg.Close();

  return true;
 }

 return false;
}

bool CMTuxvisionApp::InitInterfaces()
{
 MT_PRINT(0,"Loading interfaces...");

 CVideoPrevWnd * pWnd = MT_GetVideoPrevWnd();

 m_hrLastResult = OpenInterface(pWnd->GetSafeHwnd(), m_hInstance);
 if(FAILED(m_hrLastResult))
	 return false;

 if(!g_DBoxComm.Init())
	 return false;

 char szIP[16];
 sprintf(szIP,"%i.%i.%i.%i",g_sGlobals.m_byDreamboxIP[0],
						    g_sGlobals.m_byDreamboxIP[1],
						    g_sGlobals.m_byDreamboxIP[2],
						    g_sGlobals.m_byDreamboxIP[3]);

 if(!g_DBoxComm.SetAddress(szIP))
	 return false;

 if(!g_DBoxComm.SetUsername(g_sGlobals.m_szUsername))
	 return false;

 if(!g_DBoxComm.SetPassword(g_sGlobals.m_szPassword))
	 return false;
 
 return true;
}

char *CMTuxvisionApp::GetWin32ErrorMessage()
{
 LPVOID lpMsgBuf;

 FormatMessage( 
    FORMAT_MESSAGE_ALLOCATE_BUFFER | 
    FORMAT_MESSAGE_FROM_SYSTEM | 
    FORMAT_MESSAGE_IGNORE_INSERTS,
    NULL,
    GetLastError(),
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    (LPTSTR) &lpMsgBuf,
    0,
    NULL);
 
 return (char*)lpMsgBuf;
}

bool CMTuxvisionApp::GetChannelInfo()
{
 char *pHTML = g_DBoxComm.GetChannelInfo();
 if(pHTML)
 {
  int iLen = strlen(pHTML);

  char *pAPID = MT_GetValueInHTMLTag(pHTML,"APID",iLen);
  if(pAPID)
	strncpy(g_sGlobals.m_szAPid, pAPID, 9);

  char *pVPID = MT_GetValueInHTMLTag(pHTML,"VPID",iLen);
  if(pVPID)
	strncpy(g_sGlobals.m_szVPid, pVPID, 9);

  char *pFormat = MT_GetValueInHTMLTag(pHTML,"Format",iLen);
  if(pFormat)
	strncpy(g_sGlobals.m_szFormat, pFormat, 14);

  delete [] pHTML;

  return true;
 }

 return false;
}

bool CMTuxvisionApp::StartPreview()
{
 MT_PRINT(0,"CMTuxvisionApp::StartPreview - Starting preview sequence");

 if(g_sGlobals.m_dbState > StateStopped)	
 {
  StopGraph();
 };

 CreatePreviewGraph();

 MT_PRINT(0,"CMTuxvisionApp::StartPreview - Running preview graph");
 m_hrLastResult = RunGraph(gpIGraphBuilder);
 if(SUCCEEDED(m_hrLastResult))
 {
  g_sGlobals.m_dbState = StatePreview;
//  SetDSoundVolume(gDSoundVoume);
//  SetEvent(songevents[2]);
  //!!BS: this looks like a potential race condition, so let the thread fire event_0 !
  //SetEvent(songevents[0]);
  return true;
 }
 
 StopGraph();
 
 return false;
}

bool CMTuxvisionApp::StartRecord()
{
 if(g_sGlobals.m_dbState == StatePreview)
 {
  StopGraph();
 }

 CreateCaptureGraph();

 m_hrLastResult = RunGraph(gpIGraphBuilder);
 if(SUCCEEDED(m_hrLastResult))
 {
  g_sGlobals.m_dbState = StateRecord;
  /*SetDSoundVolume(gDSoundVoume);
    SetEvent(songevents[2]);
    //!!BS: this looks like a potential race condition, so let the thread fire event_0 !
    //SetEvent(songevents[0]);*/
  return true;
 }


 StopGraph();
 
 return false;
}

void CMTuxvisionApp::StopGraph()
{
 MT_PRINT(0,"CMTuxvisionApp::StopGraph - Stopping previously started graph");

 ::StopGraph(gpIGraphBuilder);
 DestroyGraph(gpIGraphBuilder);

 g_sGlobals.m_dbState = StateStopped;
}

bool CMTuxvisionApp::LoadLanguage(char *pszName,bool bShowConfirm)
{
 TiXmlDocument XmlFile;
 TiXmlElement  *pRootList; 

 if(!XmlFile.LoadFile(pszName))
	 return false;

 pRootList = XmlFile.FirstChildElement("language");
 if(!pRootList)
	 return false;

 // -> Translate everything

 CMenu *pMenu = m_pMainWnd->GetMenu(); 
 
 /* File Menu */
 TiXmlElement * pTemp = pRootList->FirstChildElement("file");
 if(pTemp)
 {
  // -> Top Level File
  pMenu->ModifyMenu(0,MF_BYPOSITION,0, pTemp->Attribute("val"));

  // -> Sub of File menu
  pMenu->ModifyMenu(ID_FILE_UPDATE,MF_BYCOMMAND, ID_FILE_UPDATE, pTemp->FirstChildElement("e1")->Attribute("val"));
  pMenu->ModifyMenu(ID_FILE_PREFERENCES,MF_BYCOMMAND, ID_FILE_PREFERENCES, pTemp->FirstChildElement("e3")->Attribute("val"));
  pMenu->ModifyMenu(ID_FILE_CHECKFORUPDATES,MF_BYCOMMAND, ID_FILE_CHECKFORUPDATES, pTemp->FirstChildElement("e4")->Attribute("val"));
  pMenu->ModifyMenu(ID_APP_EXIT,MF_BYCOMMAND, ID_APP_EXIT, pTemp->FirstChildElement("e5")->Attribute("val"));
 
  // -> We need a pointer to File CMenu class in order to translate "Language"
  pMenu->GetSubMenu(0)->ModifyMenu(1,MF_BYPOSITION, 0, pTemp->FirstChildElement("e2")->Attribute("val"));
 }else
	 MT_PRINT(0,"CMTuxvisionApp::LoadLanguage - Unable to translate File menu. Corrupted language file.");

 /* Help Menu */
 pTemp = pRootList->FirstChildElement("help");
 if(pTemp)
 {
  // -> Top Level File
  pMenu->ModifyMenu(1,MF_BYPOSITION,0, pTemp->Attribute("val"));

  // -> Sub of File menu
  pMenu->ModifyMenu(ID_HELP_LOG,MF_BYCOMMAND, ID_HELP_LOG, pTemp->FirstChildElement("e1")->Attribute("val"));
  pMenu->ModifyMenu(ID_HELP_MANUAL,MF_BYCOMMAND, ID_HELP_MANUAL, pTemp->FirstChildElement("e2")->Attribute("val"));
  pMenu->ModifyMenu(ID_APP_ABOUT,MF_BYCOMMAND, ID_APP_ABOUT, pTemp->FirstChildElement("e3")->Attribute("val"));
 }else
	 MT_PRINT(0,"CMTuxvisionApp::LoadLanguage - Unable to translate Help menu. Corrupted language file.");

 // -> Report some informations
 char szReport[255];

 if(bShowConfirm)
 {
  sprintf(szReport,"Language\t%s\nAuthor\t%s\nVersion\t%s\nContact\t%s\n\nNot yet enabled.",
		pszName,
		((TiXmlElement*)pRootList->FirstChildElement("header"))->Attribute("author"),
		((TiXmlElement*)pRootList->FirstChildElement("header"))->Attribute("version"),
		((TiXmlElement*)pRootList->FirstChildElement("header"))->Attribute("contact"));

  AfxMessageBox(szReport);
 }

 MT_PRINT(1,"Language file %s information:\r\nAuthor\t%s\r\nVersion\t%s\r\nContact\t%s",
		pszName,
		((TiXmlElement*)pRootList->FirstChildElement("header"))->Attribute("author"),
		((TiXmlElement*)pRootList->FirstChildElement("header"))->Attribute("version"),
		((TiXmlElement*)pRootList->FirstChildElement("header"))->Attribute("contact"));

 return true;
}

// App command to run the dialog
void CMTuxvisionApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CMTuxvisionApp message handlers


int CMTuxvisionApp::ExitInstance() 
{
	SaveGlobals();

	MT_PRINT(0,"Shutting down");	

	if(g_sGlobals.m_dbState > StateStopped)	
	{
	 StopGraph();
	};

	CoUninitialize();
	
	return CWinApp::ExitInstance();
}
