/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#include "stdafx.h"
#include "MTuxvision.h"

#include "MainFrm.h"
#include "MTuxvisionDoc.h"
#include "MTSchedule.h"
#include "MTuxvisionView.h"
#include "MTBouquetsView.h"
#include "MTChannelView.h"
#include "preferences.h"
#include <math.h>
#include <io.h>

#include "Engine\dboxcomm.h"
#include <dshow.h>
#include "Engine\DShow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MT_SPLITTER_PANE_WIDTH	175

__int64 g_FilteredAudioBitrate = 0;
__int64 g_FilteredVideoBitrate = 0;

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SIZING()
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	ON_MESSAGE(WM_STREAMNOTIFY, OnStreamNotify)
	ON_WM_TIMER()
	ON_COMMAND(ID_PREFERENCES, OnPreferences)
	ON_COMMAND(ID_RECORD, OnRecord)
	ON_COMMAND(ID_STOP, OnStop)
	ON_COMMAND(ID_PREVIEW, OnPreview)
	ON_COMMAND(ID_UPDATELIST, OnUpdatelist)
	ON_UPDATE_COMMAND_UI(ID_STOP, OnUpdateStop)
	ON_UPDATE_COMMAND_UI(ID_RECORD, OnUpdateRecord)
	ON_COMMAND(ID_HELP_LOG, OnHelpLog)
	ON_UPDATE_COMMAND_UI(ID_PREVIEW, OnUpdatePreview)
	ON_COMMAND(ID_FILE_PREFERENCES, OnPreferences)
	ON_COMMAND(ID_FILE_UPDATE, OnUpdatelist)
	ON_UPDATE_COMMAND_UI(ID_FILE_UPDATE, OnUpdateFileUpdate)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_LANG_01, 
                   ID_LANG_10, 
                   OnApplyTranslation) 
END_MESSAGE_MAP()

static UINT PaneIndicators[] =
{
	ID_SEPARATOR,           // Name (SOME WEIRD ID!?)
	ID_SEPARATOR,			// Vol
	ID_SEPARATOR,			// Video
	ID_SEPARATOR,			// Audio
	ID_SEPARATOR,			// Undefined
};

static UINT PaneSizes[] =
{
	20,	
	20,
	20,
	20,
	110
};

#define MT_NUM_PANES	5


/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
 m_bPostInit = false;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT|TBSTYLE_LIST, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if(!m_wndToolBar.LoadBitmap(MAKEINTRESOURCE(IDB_TOOLBAR)))
	{
		TRACE0("Failed to load toolbar bitmap");
		return -1;
	}	

	SetupToolbar();

	RecalcLayout(TRUE);

	if(!CreateStatusBar())
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// -> Write out some dummy text to our status bar
	m_wndStatusBar.SetPaneText(0,"No channel selected"); 
	//m_wndStatusBar.SetPaneText(1,"- III....... +"); 
	m_wndStatusBar.SetPaneText(2,"Audio : 1256 kb/s (0xFFFF)"); 
	m_wndStatusBar.SetPaneText(3,"Video : 1200 kb/s (0xAF01)"); 

	m_uTimer = SetTimer(1,1500,NULL);

	BuildAndSetTitle();

	SetIcon(NULL,FALSE);
	SetIcon(NULL,TRUE);

	/* Initialize fullscreen window */
	bool b = m_wndFullscreen.Create();

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style &= ~FWS_ADDTOTITLE;

	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

void CMainFrame::SetupToolbar()
{
 UINT uTemp = 0;
 int iImg = 0,iBtnIndex = 0;

 CBitmap bmp;
 bmp.Attach(LoadBitmap(GetModuleHandle(NULL),MAKEINTRESOURCE(IDB_TOOLBAR)));

 CImageList *pImageList = new CImageList();
 
 pImageList->Create(40,40,ILC_MASK|ILC_COLOR8,5,1);
 //pImageList->Create(MAKEINTRESOURCE(IDB_TOOLBAR),40, 5, RGB(192,192,192));
 pImageList->SetBkColor(CLR_NONE);
 
 pImageList->Add(&bmp,RGB(192,192,192));

 m_wndToolBar.GetToolBarCtrl().SetImageList(pImageList);

// m_wndToolBar.SetBitmap(LoadBitmap(GetModuleHandle(NULL),MAKEINTRESOURCE(IDB_TOOLBAR)));


 /*
 m_wndToolBar.SetSizes(CSize(50,50),CSize(40,40));*/

 // -> Preview button is an on off
 iBtnIndex = m_wndToolBar.CommandToIndex(ID_PREVIEW);
 m_wndToolBar.GetButtonInfo(iBtnIndex,uTemp,uTemp,iImg);
 m_wndToolBar.SetButtonInfo(iBtnIndex,ID_PREVIEW,TBBS_CHECKBOX,iImg);

 m_wndToolBar.GetToolBarCtrl().SetState(ID_PREVIEW,TBSTATE_CHECKED|TBSTATE_ENABLED);
}

bool CMainFrame::CreateStatusBar()
{
 if(!m_wndStatusBar.Create(this)) 
	return false;

 if(!m_wndStatusBar.SetIndicators(PaneIndicators, MT_NUM_PANES))
	return false;

 // -> Create our volume slider 
 if(!m_wndSliderVolume.Create(WS_CHILD|WS_VISIBLE|TBS_AUTOTICKS|TBS_HORZ, CRect(0,0,10,10), &m_wndStatusBar,IDC_VOLSLIDER))
	 return false;

 m_wndSliderVolume.SetRangeMin(0);
 m_wndSliderVolume.SetRangeMax(100);
 m_wndSliderVolume.SetTicFreq(10);
 m_wndSliderVolume.SetPos(g_sGlobals.m_byVolume);

 ResizeStatusBar();

 return true;
}

void CMainFrame::ResizeStatusBar()
{
 if(m_wndStatusBar.GetSafeHwnd() == NULL)
	 return;

 UINT uCurPaneIndex = 0;
 UINT uPaneStyle = 0;
 int iTemp;
 CRect cRect;

 GetClientRect(&cRect);

 float fTemp = cRect.Width() / 100.0f;
 int iSize = 0;

 for(int i = 0; i < MT_NUM_PANES; i++)
 { 
  m_wndStatusBar.GetPaneInfo(i,uCurPaneIndex,uPaneStyle,iTemp);

  iSize = (PaneSizes[i] > 100) ? (PaneSizes[i] - 100) : ((int)floorf(fTemp * PaneSizes[i]));

  switch(i)
  {
  case 0:
	m_wndStatusBar.SetPaneInfo(i,uCurPaneIndex,SBPS_NORMAL | SBPS_STRETCH,iSize);
	break;
  case MT_STS_VOLCONTROL_PANE:
	  {
	   m_wndStatusBar.SetPaneInfo(i,uCurPaneIndex,uPaneStyle | SBT_OWNERDRAW | SBT_NOBORDERS,iSize);
	
	   // -> Calculate slider rect
	   CRect rect;
	   m_wndStatusBar.GetItemRect(i,rect);

	   // If we've got enougth space, draw sound icon...
	   if(rect.Width() > 30)
	   {   
		rect.left  += 16;
	   }

	   m_wndSliderVolume.SetWindowPos(NULL,rect.left,rect.top,rect.Width(),rect.Height(),SWP_NOREPOSITION);
	  }break;
  case MT_STS_DBSTATE_PANE:
	m_wndStatusBar.SetPaneInfo(i,uCurPaneIndex,uPaneStyle | (SBT_OWNERDRAW | SBT_NOBORDERS),iSize);
	break;
  default:
	m_wndStatusBar.SetPaneInfo(i,uCurPaneIndex,SBPS_NORMAL,iSize);
  }
 }
}

void CMainFrame::CenterVideoPrevWindow()
{
 CVideoPrevWnd *pVideo = MT_GetVideoPrevWnd();
 if(!pVideo)
	 return;

 CMTuxvisionView *pView = MT_GetVideoView();
 if(!pView)
	 return;

 CRect viewRect,prevRect;
 pView->GetClientRect(&viewRect);
 pVideo->GetWindowRect(&prevRect);
 
 int iXCenter = viewRect.Width() >> 1;
 int iXOffset = iXCenter - (prevRect.Width() >> 1);

 int iYCenter = viewRect.Height() >> 1;
 int iYOffset = iYCenter - (prevRect.Height() >> 1);

 pVideo->MoveWindow(iXOffset,iYOffset,prevRect.Width(),prevRect.Height());
}

void CMainFrame::BuildAndSetTitle()
{
 char szTemp[255];

 switch(g_sGlobals.m_dbState)
 {
 case StateUninitialized:
	sprintf(szTemp,"MTuxvision %.1f Build %i",MTUXVISION_VERSION,MTUXVISION_BUILD);
	break;
 case StateStopped:
	sprintf(szTemp,"MTuxvision %.1f Build %i (Stopped)",MTUXVISION_VERSION,MTUXVISION_BUILD);
	break;
 case StatePreview:
	sprintf(szTemp,"MTuxvision %.1f Build %i (Previewing...)",MTUXVISION_VERSION,MTUXVISION_BUILD);
	break;
 case StateRecord:
	sprintf(szTemp,"MTuxvision %.1f Build %i (Recording...)",MTUXVISION_VERSION,MTUXVISION_BUILD);
	break;
 }

 SetWindowText(szTemp);
}

void CMainFrame::BuildLangMenu()
{
 struct _finddata_t c_file;
 long hFile;

 char szMask[255];
 sprintf(szMask,"%s\\lang\\*.xml",MT_GetApp()->m_szAppPath);

 CMenu *pTop = GetMenu();
 CMenu *pFile = pTop->GetSubMenu(0);
 CMenu *pLangMenu = pFile->GetSubMenu(1);

 // -> Let's empty our menu
 for(UINT i = 0; i < pLangMenu->GetMenuItemCount(); i++)
	pLangMenu->RemoveMenu(i,MF_BYPOSITION);

 // -> Fill it with langauges 10 languages...
 int iIndex = 0;
 hFile = _findfirst(szMask,&c_file);
 if(hFile == -1L)
 {
  pLangMenu->InsertMenu(-1,MF_BYPOSITION,ID_LANG_01,"No languages");
  MT_PRINT(0,"CMainFrame::BuildLangMenu - No custom translation found");
  return;
 }

 char szTemp[25];
 int iLen = 0;

 do
 {
  iLen = strlen(c_file.name) - 4;
  strncpy(szTemp,c_file.name, iLen);
  szTemp[iLen] = '\0';
  
  pLangMenu->InsertMenu(-1,MF_BYPOSITION,ID_LANG_01 + iIndex, szTemp);
  
  iIndex++;
 } while((_findnext(hFile,&c_file) == 0) && iIndex <= 10);
}

void CMainFrame::OnVolumeNotify()
{
 g_sGlobals.m_byVolume = (BYTE)m_wndSliderVolume.GetPos();

 SetDSoundVolume((__int64)g_sGlobals.m_byVolume);

 m_wndStatusBar.InvalidateRect(NULL,FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnSizing(UINT fwSide, LPRECT pRect) 
{
	CFrameWnd::OnSizing(fwSide, pRect);
	
	ResizeStatusBar();
}

void CMainFrame::OnSize(UINT nType, int cx, int cy) 
{
	CFrameWnd::OnSize(nType, cx, cy);
	
    if(m_bPostInit && nType != SIZE_MINIMIZED)
	{
     int iMinWidth = 150;

     m_wndSplitter.SetRowInfo( 0, cy, 0 );

	 if(cx - (2*MT_SPLITTER_PANE_WIDTH) < iMinWidth)
		m_wndSplitter.SetColumnInfo( 1, iMinWidth , 0);
     else
		m_wndSplitter.SetColumnInfo( 1, cx - (3*MT_SPLITTER_PANE_WIDTH), 0);
	 
	 m_wndSplitter.SetColumnInfo( 0, MT_SPLITTER_PANE_WIDTH, 0 );
	 m_wndSplitter.SetColumnInfo( 2, MT_SPLITTER_PANE_WIDTH, 0 );
     m_wndSplitter.SetColumnInfo( 3, MT_SPLITTER_PANE_WIDTH, 0 );

     m_wndSplitter.RecalcLayout();

	 ResizeStatusBar();
	 CenterVideoPrevWindow();
	}	
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	bool bret = (CFrameWnd::OnCreateClient(lpcs, pContext) != 0);

	if(!m_wndSplitter.CreateStatic(this,1,4))
		return FALSE;

	SIZE previewSize = { 250, 200 };
	SIZE minSize = {175,200};

	m_wndSplitter.CreateView(0,0,RUNTIME_CLASS(CMTSchedule),minSize,pContext);
	m_wndSplitter.CreateView(0,1,RUNTIME_CLASS(CMTuxvisionView),previewSize,pContext);
	m_wndSplitter.CreateView(0,2,RUNTIME_CLASS(CMTChannelView),minSize,pContext);
	m_wndSplitter.CreateView(0,3,RUNTIME_CLASS(CMTBouquetsView),minSize,pContext);
	
	m_bPostInit = true;

	return true;
}

void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	lpMMI->ptMinTrackSize.x = 950;
	lpMMI->ptMinTrackSize.y = 500;

	CFrameWnd::OnGetMinMaxInfo(lpMMI);
}

LRESULT CMainFrame::OnStreamNotify(WPARAM wParam,LPARAM lParam)
{
 AfxMessageBox("STRAM NOTIFY");

 return 0;
}

void CMainFrame::OnTimer(UINT nIDEvent) 
{
/* // -> That's not timer we want.
 if(nIDEvent != m_uTimer)
	return;
*/
 // -> We are not recording nor previewing.
 if(g_sGlobals.m_dbState <= StateStopped)
 {
  m_wndStatusBar.SetPaneText(2,"Audio : No signal"); 
  m_wndStatusBar.SetPaneText(3,"Video : No signal"); 
  return;
 };

 __int64 i64Val			= 0;
 __int64 i64AVStatus	= 0;

 char szString[255];
 
 if(!g_DBoxComm.GetResyncCountAndStatus(&i64Val, &i64AVStatus))
 {
  MT_PRINT(0,"CMainFrame::OnTimer - Unable to get resync count");
  return;
 }

 __int64 i64AudioRate = 0;
 __int64 i64VideoRate = 0;
 
 if(!g_DBoxComm.GetAVBitrates(&i64AudioRate, &i64VideoRate))
	 return;

 // -> Audio first
 g_FilteredAudioBitrate = (g_FilteredAudioBitrate*8 + i64AudioRate*2)/10;
 i64AudioRate = g_FilteredAudioBitrate / 1024;

 sprintf(szString,"Audio: %I64d kBit/s (%s)",i64AudioRate,g_sGlobals.m_szAPid);
 m_wndStatusBar.SetPaneText(2,szString); 

 // -> Video then
 g_FilteredVideoBitrate = (g_FilteredVideoBitrate*8 + i64VideoRate*2)/10;
 i64VideoRate = g_FilteredVideoBitrate / 1024;

 sprintf(szString,"Video: %I64d kBit/s (%s)",i64VideoRate,g_sGlobals.m_szVPid);
 m_wndStatusBar.SetPaneText(3,szString); 

 CFrameWnd::OnTimer(nIDEvent);
}

void CMainFrame::OnPreferences() 
{
 CPreferences dlgPref("Preferences", this);

 dlgPref.DoModal();
}

void CMainFrame::OnRecord() 
{
 MT_GetApp()->StartRecord();

 BuildAndSetTitle();
}

void CMainFrame::OnStop() 
{
 if(g_sGlobals.m_dbState > StateStopped)
	 MT_GetApp()->StopGraph();

 BuildAndSetTitle();
}

void CMainFrame::OnPreview() 
{
 g_sGlobals.m_bRecAndPreview = !g_sGlobals.m_bRecAndPreview;

 if(g_sGlobals.m_dbState == StatePreview)
	 MT_GetApp()->StopGraph();
 else
	 MT_GetApp()->StartPreview();

 CMTuxvisionView *pView = MT_GetVideoView();
 if(pView)
	 pView->InvalidateRect(NULL,FALSE);
}

void CMainFrame::OnUpdatelist() 
{
 int iRes = AfxMessageBox("Are you sure you want to reload bouquets list from dreambox?", MB_YESNO|MB_ICONQUESTION);
 if(iRes != IDYES)
	return;

 CTreeCtrl *pTree = MT_GetBouquetsCtrl();
 CListCtrl *pList = MT_GetChanCtrl();

 CBouquetsReader *pBouquets = &MT_GetApp()->m_cBouquetsReader;	
 if(!pBouquets)
 {
  MT_PRINT(0,"CMainFrame::OnUpdatelist - Error while looking for bouquets reader");
  AfxMessageBox("Impossible to find bouquets reader");
  return;
 }

 // -> Clean up. Duh.
 pBouquets->Destroy();
 pBouquets->Clear();

 pTree->DeleteAllItems();
 pList->DeleteAllItems();

 // -> Reload.
 CMTBouquetsView *pView = (CMTBouquetsView *)((CMainFrame*)MT_GetApp()->m_pMainWnd)->m_wndSplitter.GetPane(0,3);
 if(!pView)
	 return;

 if(pView->LoadBouquets())
	AfxMessageBox("Bouquets downloaded and reloaded correctly!",MB_ICONASTERISK);
}

void CMainFrame::OnUpdateStop(CCmdUI* pCmdUI) 
{
 if(g_sGlobals.m_dbState == StateRecord)
 {
  pCmdUI->Enable();
  return;
 }

 pCmdUI->Enable(FALSE);
}

void CMainFrame::OnUpdateRecord(CCmdUI* pCmdUI) 
{
 if(g_sGlobals.m_dbState != StateRecord)
 {
  pCmdUI->Enable();
  return;
 }

 pCmdUI->Enable(FALSE);	
}

void CMainFrame::OnHelpLog() 
{
 char szLogFile[255];
 sprintf(szLogFile,"%s/mtuxvision.txt",MT_GetApp()->m_szAppPath);

 ShellExecute(GetSafeHwnd(), "Open", szLogFile, NULL, MT_GetApp()->m_szAppPath, SW_SHOW);
}

void CMainFrame::OnUpdatePreview(CCmdUI* pCmdUI) 
{
 if(g_sGlobals.m_dbState != StateRecord)
 {
  pCmdUI->Enable();
  return;
 }

 pCmdUI->Enable(FALSE);		
}

void CMainFrame::OnUpdateFileUpdate(CCmdUI* pCmdUI) 
{
 // -> When something gets updated, update avaiable langauges
 BuildLangMenu();
}

void CMainFrame::OnApplyTranslation(UINT uID)
{
 int iRes = AfxMessageBox("This will apply the selected translation. Continue?",MB_YESNO|MB_ICONQUESTION);
 if(iRes != IDYES)
	 return;

 char szString[25]; 
 GetMenu()->GetMenuString(uID, szString, 24, MF_BYCOMMAND);

 char szFileName[255];
 sprintf(szFileName,"%s\\lang\\%s.xml",MT_GetApp()->m_szAppPath, szString);

 if(!MT_GetApp()->LoadLanguage(szFileName))
	 return;//MT_GetApp()->LoadDefaultLanguage();

 strcpy(g_sGlobals.m_szLanguage, szString);
}
