/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/

#ifndef __MUTEX_READERS_WRITERS
#define __MUTEX_READERS_WRITERS

#include <assert.h>

class CMutexRW
{
protected:
	HANDLE		m_semReaders;
	HANDLE		m_semWriters;
	int			m_nReaders;
public:
	CMutexRW() :
		m_semReaders(NULL),
		m_semWriters(NULL),
		m_nReaders(0)
	{
		// initialize the Readers & Writers variables
		m_semReaders	= ::CreateSemaphore(NULL, 1, 1, NULL);
		m_semWriters	= ::CreateSemaphore(NULL, 1, 1, NULL);
		m_nReaders		= 0;

		if (m_semReaders == NULL || m_semWriters == NULL)
		{
			LPVOID lpMsgBuf;
			FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL 
				);
			printf( "***** ERROR: CreateSemaphore: %s\n", (LPCTSTR)lpMsgBuf );
			LocalFree( lpMsgBuf );			
		}
	};

	virtual ~CMutexRW()
	{
		if (m_semWriters)
			::CloseHandle(m_semWriters);	

		m_semWriters = NULL;
		if (m_semReaders)
			::CloseHandle(m_semReaders);	
		m_semReaders = NULL;
	}

	inline void Lock_DataRead(){
		DWORD dwEvent = WAIT_TIMEOUT;

		// P( semReaders )
		dwEvent = ::WaitForSingleObject( m_semReaders, INFINITE );
		assert(dwEvent == WAIT_OBJECT_0);

		m_nReaders++;

		if (m_nReaders == 1)
		{
			// P( semWriters )
			dwEvent = ::WaitForSingleObject( m_semWriters, INFINITE );
			assert(dwEvent == WAIT_OBJECT_0);
		}
		// V( semReaders )
		::ReleaseSemaphore( m_semReaders, 1, NULL );
	};

	inline void Unlock_DataRead(){
		DWORD dwEvent = WAIT_TIMEOUT;
		// P( semReaders )
		dwEvent = ::WaitForSingleObject( m_semReaders, INFINITE );
		assert(dwEvent == WAIT_OBJECT_0);

		m_nReaders--;

		if (m_nReaders == 0)
		{
			// V( semWriters )
			::ReleaseSemaphore(m_semWriters, 1, NULL);
		}
		// V( semReaders )
		::ReleaseSemaphore( m_semReaders, 1, NULL );
	};

	inline void Lock_DataWrite(){
		DWORD dwEvent = WAIT_TIMEOUT;

		// P( semWriters )
		dwEvent = ::WaitForSingleObject( m_semWriters, INFINITE );
		assert(dwEvent == WAIT_OBJECT_0);
	}

	inline void Unlock_DataWrite(){
		// V( semWriters )
		::ReleaseSemaphore(m_semWriters, 1, NULL);
	};

};

class CReadLock
{
protected:
	CMutexRW*	m_pMutexRW;
	bool		m_bIsLocked;
public:
	CReadLock(CMutexRW* pMutexRW, const bool bInitialLock = false) :
		m_pMutexRW(pMutexRW), m_bIsLocked(false)
	{
		assert(m_pMutexRW);
		if (bInitialLock){
			m_pMutexRW->Lock_DataRead();
			m_bIsLocked = true;
		}
	};

	inline const bool& IsLocked() const{
		return m_bIsLocked;
	};

	inline void Lock(){
		assert(m_bIsLocked == false);
		m_pMutexRW->Lock_DataRead();
		m_bIsLocked = true;
	};

	inline void Unlock(){
		assert(m_bIsLocked);
		m_pMutexRW->Unlock_DataRead();
		m_bIsLocked = false;
	};
	virtual ~CReadLock(){
		if (m_bIsLocked){
			m_pMutexRW->Unlock_DataRead();
		}
	};
};

class CWriteLock
{
protected:
	CMutexRW*	m_pMutexRW;
	bool		m_bIsLocked;
public:
	CWriteLock(CMutexRW* pMutexRW, const bool bInitialLock = false) :
		m_pMutexRW(pMutexRW), m_bIsLocked(false)
	{
		assert(m_pMutexRW);
		if (bInitialLock){
			m_pMutexRW->Lock_DataWrite();
			m_bIsLocked = true;
		}
	};

	inline const bool& IsLocked() const{
		return m_bIsLocked;
	};

	inline void Lock(){
		assert(m_bIsLocked == false);
		m_pMutexRW->Lock_DataWrite();
		m_bIsLocked = true;
	};

	inline void Unlock(){
		assert(m_bIsLocked);
		m_pMutexRW->Unlock_DataWrite();
		m_bIsLocked = false;
	};
	virtual ~CWriteLock(){
		if (m_bIsLocked){
			m_pMutexRW->Unlock_DataWrite();
		}
	};
};

#endif // end #ifndef __MUTEX_READERS_WRITERS
