/*
  MTuxvision - View and record channels from your Dreambox to your PC
  Copyright (C) 2007  Alessio Pierluigi Placitelli (a.placitelli@a2p.it)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
                      !!!!!! IMPORTANT !!!!!! 
  This software is based on 2004 Bernd Scharping's "tuxvision", which was
  released under the terms of GPL v2 license. Bernd "BSE" Scharping, trgically
  died in a mid-2003 car accident (http://wiki.tuxbox.org/TuxVision). Thank
  you for great software Mr. Scharping    

  ***************************************************************************
    
  File description: -
  
  Todo:
    -  
  
  Bugs:   
    -
*/


#include "stdafx.h"
#include "mtuxvision.h"
#include "Parser.h"

CParser	g_strParser;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CParser::CParser()
{

}

CParser::~CParser()
{
 m_pBuffer		= NULL;
 m_pTokenStart  = NULL;
}

/* Little note about Start/StartFile.
 
   If you need to load the buffer directly from a file, use StartFile. It
   will take care of everything.

   If you need to use parser ONCE on a memory buffer, use Start without 
   specifying the iLen parameter (len of the buffer) or setting it to -1

   If you need to parse multiple times same buffer, calculate buffer len 
   BEFORE calling GetNextToken for the first time because GetNextToken fills
   the buffer with null characters in place of separators so you wont be able
   to retrieve buffer len because it is computed from 0 to first NULL char
*/
char *CParser::StartFile(const char *pszFileName, const char *pszTokens)
{
 FILE *p = fopen(pszFileName,"rt");
 if(!p)
	 return NULL;

 fseek(p,0,SEEK_END);
 long lSize = ftell(p);
 fseek(p,0,SEEK_SET);

 char *pszBuff = new char[lSize];
 if(!pszBuff)
 {
  fclose(p);
  return NULL;
 }

 fread(pszBuff,1,lSize,p);

 fclose(p);

 if(!Start(pszBuff,pszTokens))
 {
  delete [] pszBuff;
  return NULL;
 }

 return pszBuff;
}

bool CParser::Start(const char *pszBuff, const char *pszTokens)
{
 if(!pszBuff)
	 return false;

 m_pTokenStart  = NULL;

 m_uBufferSize 	= strlen(pszBuff);
 m_uNumTokens	= strlen(pszTokens);

 m_pBuffer		= (char*)pszBuff;
 strcpy(m_szTokens, pszTokens); 

 return true;
}

bool CParser::Start(const char * pBuff, int iBuffLen, const char *pszTokens)
{
 if(!pBuff)
	 return false;

 m_pTokenStart  = NULL;

 m_uBufferSize 	= iBuffLen;
 m_uNumTokens	= strlen(pszTokens);

 m_pBuffer		= (char*)pBuff;
 strcpy(m_szTokens, pszTokens); 

 return true;
}

char *CParser::GetNextToken()
{
// m_pTokenStart = NULL;

 do
 {
  if(IsToken(*m_pBuffer))	
  {
   // Skip first time use...
   char *tempPtr = m_pTokenStart;
   
   // Append null character so user wont get fu*ked
   *m_pBuffer = '\0';

   // Jump to next character
   Step();

   // If we reached another separator, return found token!
   m_pTokenStart = m_pBuffer;

   // Dont return whitespaces
   if(tempPtr && !iswspace(*tempPtr))//(strlen(tempPtr) > 1))
	return tempPtr;
   else	// if we dont handle this else, it will skip 1 character if there are two consecutive separators
	   continue;
  }

  // -> Memorize beginning of the token for the first time
  if(!m_pTokenStart)
	  m_pTokenStart = m_pBuffer;

  // Jump to next character
  Step();
 
 }while(m_uBufferSize);

 // NEVER unless buffer end.
 return NULL;
}

bool CParser::IsToken(char c)
{
 if(c == '\0')
	 return true;	// Automatically skip whitespaces

 for(UINT u = 0; u < m_uNumTokens; u++)
 {
  if(m_szTokens[u] == c)
	  return true;
 }

 return false;
}

void CParser::Step()
{
 if(m_uBufferSize == 0)
	 return;

 m_pBuffer++;
 --m_uBufferSize; 
}
